@@ -5,7 +5,7 @@
 ;; Author: Protesilaos Stavrou <info@protesilaos.com>
 ;; Maintainer: Protesilaos Stavrou <info@protesilaos.com>
 ;; URL: https://github.com/protesilaos/dired-preview
-;; Version: 0.4.0
+;; Version: 0.5.0
 ;; Package-Requires: ((emacs "28.1"))
 ;; Keywords: files, convenience
 
@@ -72,7 +72,7 @@
   (concat "\\."
           "\\(mkv\\|webm\\|mp4\\|mp3\\|ogg\\|m4a\\|flac\\|wav"
           "\\|gz\\|zst\\|tar\\|xz\\|rar\\|zip"
-          "\\|iso\\|epub\\|pdf\\)")
+          "\\|iso\\|epub\\|pdf\\)\\'")
   "Regular expression of file type extensions to not preview.
 When the value is nil, do not ignore any file: preview
 everything.
@@ -174,7 +174,9 @@ times the height of the frame."
   :risky t)
 
 (defcustom dired-preview-delay 0.7
-  "Time in seconds to wait before previewing."
+  "Time in seconds to wait before previewing.
+If the value is 0, then it is internally understood as 0.1 as no delay
+can affect performance."
   :group 'dired-preview
   :type 'number)
 
@@ -282,25 +284,28 @@ aforementioned user option."
   (seq-filter #'dired-preview--window-parameter-p (window-list)))
 
 (defun dired-preview--delete-windows ()
-  "Delete preview windows."
+  "Delete preview windows or clean them up if they should not be deleted."
   (dolist (window (dired-preview--get-windows))
-    (when (and (not (one-window-p))
+    (if (and (not (one-window-p))
                (window-live-p window)
-               (not (eq window (minibuffer-window))))
-      (delete-window window))))
+               (not (eq window (minibuffer-window)))
+               (not (window-prev-buffers window)))
+        (delete-window window)
+      (dired-preview--clean-up-window window))))
 
 (defun dired-preview--file-ignored-p (file)
   "Return non-nil if FILE extension is among the ignored extensions.
 See user option `dired-preview-ignored-extensions-regexp'."
-  (when-let* (((not (file-directory-p file)))
-              ((stringp dired-preview-ignored-extensions-regexp))
-              (ext (file-name-extension file :include-dot))
-              ((not (string-blank-p ext))))
-    (string-match-p ext dired-preview-ignored-extensions-regexp)))
+  (when-let* (((stringp dired-preview-ignored-extensions-regexp))
+              ((not (file-directory-p file)))
+              (file-nondir (file-name-nondirectory file)))
+    (string-match-p dired-preview-ignored-extensions-regexp file-nondir)))
 
 (defun dired-preview--file-large-p (file)
   "Return non-nil if FILE exceeds `dired-preview-max-size'."
-  (>= (file-attribute-size (file-attributes file)) dired-preview-max-size))
+  (>= (or (file-attribute-size (file-attributes file))
+          0)
+      dired-preview-max-size))
 
 (defun dired-preview--file-displayed-p (file)
   "Return non-nil if FILE is already displayed in a window."
@@ -315,36 +320,32 @@ See user option `dired-preview-ignored-extensions-regexp'."
     (set-window-parameter window 'dedicated value)
     (set-window-parameter window 'no-other-window value)))
 
-(defun dired-preview--clean-up-window ()
-  "Delete or clean up preview window."
-  (let* ((window (selected-window))
-         (buffer (window-buffer window)))
-    (if (window-parameter window 'dired-preview-window)
-        (dired-preview--delete-windows)
-      (dired-preview--rename-buffer (window-buffer window) :make-public)
-      (setq dired-preview--buffers (delq buffer dired-preview--buffers))
-      (dired-preview--set-window-parameters window nil)
-      (remove-hook 'post-command-hook #'dired-preview--clean-up-window :local))))
+(defun dired-preview--clean-up-window (&optional window)
+  "Remove preview state from WINDOW or `selected-window'."
+  (let* ((w (or window (selected-window)))
+        (buffer (window-buffer w)))
+    (dired-preview--rename-buffer (window-buffer w) :make-public)
+    (setq dired-preview--buffers (delq buffer dired-preview--buffers))
+    (dired-preview--set-window-parameters w nil)
+    (remove-hook 'post-command-hook #'dired-preview--clean-up-window :local)))
 
 ;; TODO 2024-04-22: Add PDF type and concomitant method to display its buffer.
 (defun dired-preview--infer-type (file)
   "Infer what type FILE is.
 Return a cons cell whose `car' is a symbol describing FILE and `cdr' is
 FILE."
-  (let ((ext (file-name-extension file :include-dot))
-        (file (expand-file-name file)))
+  (let* ((file (expand-file-name file))
+         (file-nondir (file-name-nondirectory file)))
     (cond
-     ((and (not (string-empty-p ext))
-           dired-preview-ignored-extensions-regexp
-           (string-match-p ext dired-preview-ignored-extensions-regexp))
+     ((and dired-preview-ignored-extensions-regexp
+           (string-match-p dired-preview-ignored-extensions-regexp file-nondir))
       (cons 'ignore file))
-     ((dired-preview--file-large-p file)
-      (cons 'large file))
-     ((and (not (string-empty-p ext))
-           (string-match-p ext dired-preview-image-extensions-regexp))
-      (cons 'image file))
      ((file-directory-p file)
       (cons 'directory file))
+     ((dired-preview--file-large-p file)
+      (cons 'large file))
+     ((string-match-p dired-preview-image-extensions-regexp file-nondir)
+      (cons 'image file))
      (t
       (cons 'text file)))))
 
@@ -382,9 +383,11 @@ FILE."
 (defmacro dired-preview-with-window (&rest body)
   "Evaluate BODY with the Dired preview window as selected."
   (declare (indent 0))
-  `(dolist (win (dired-preview--get-windows))
-     (with-selected-window win
-       ,@body)))
+  `(if-let* ((windows (dired-preview--get-windows)))
+       (dolist (win windows)
+         (with-selected-window win
+           ,@body))
+     (user-error "No dired-preview window available")))
 
 (defun dired-preview-find-file ()
   "Visit the currently previewed buffer with `find-file'.
@@ -465,15 +468,22 @@ Also see `dired-preview-find-file'."
 This technically runs `scroll-up-command'."
   (interactive)
   (dired-preview-with-window
-    (call-interactively 'scroll-up-command)))
+    (call-interactively
+     (pcase (derived-mode-p major-mode)
+       ('doc-view-mode 'doc-view-scroll-up-or-next-page)
+       ('pdf-view-mode 'pdf-view-scroll-up-or-next-page)
+       (_ 'scroll-up-command)))))
 
 ;; Same as above for the terminology.
 (defun dired-preview-page-up ()
   "Move a page up in the preview window.
 This technically runs `scroll-down-command'."
   (interactive)
-  (dired-preview-with-window
-    (call-interactively 'scroll-down-command)))
+  (call-interactively
+     (pcase (derived-mode-p major-mode)
+       ('doc-view-mode 'doc-view-scroll-down-or-previous-page)
+       ('pdf-view-mode 'pdf-view-scroll-down-or-previous-page)
+       (_ 'scroll-down-command))))
 
 (declare-function hexl-mode "hexl")
 (declare-function hexl-mode-exit "hexl" (&optional arg))
@@ -709,6 +719,15 @@ More specifically, test if FILE has an extension among the
     (put 'dired-preview-start 'function-executed t)
     (dired-preview-display-file file)))
 
+(defun dired-preview--start-idle-timer (file)
+  "Start the idle timer to preview FILE."
+  (setq dired-preview--timer
+        (run-with-idle-timer
+         (if (> dired-preview-delay 0)
+             dired-preview-delay
+           0.1)
+         nil #'dired-preview-display-file file)))
+
 (defun dired-preview-trigger (&optional no-delay)
   "Trigger display of file at point after `dired-preview-trigger-commands'.
 With optional NO-DELAY do not start a timer.  Otherwise produce
@@ -724,8 +743,7 @@ the preview with `dired-preview-delay' of idleness."
                ((and preview (memq this-command dired-preview-trigger-commands))
                 (if no-delay
                     (dired-preview-display-file file)
-                  (setq dired-preview--timer
-                        (run-with-idle-timer dired-preview-delay nil #'dired-preview-display-file file))))
+                  (dired-preview--start-idle-timer file)))
                (preview
                 (dired-preview-start file))
                ((and (not preview)
@@ -734,10 +752,17 @@ the preview with `dired-preview-delay' of idleness."
         (dired-preview--close-previews-outside-dired))
     ((error user-error quit) nil)))
 
+(defun dired-preview-get-first-window ()
+  "Return a window object for `other-window-scroll-default'."
+  (car (dired-preview--get-windows)))
+
 (defun dired-preview-disable-preview ()
   "Disable Dired preview."
   (unless (eq major-mode 'dired-mode)
     (user-error "Can only use `dired-preview' in Dired"))
+  (when (and other-window-scroll-default
+             (eq other-window-scroll-default #'dired-preview-get-first-window))
+    (setq-local other-window-scroll-default nil))
   (remove-hook 'post-command-hook #'dired-preview-trigger :local)
   (dired-preview--close-previews)
   (put 'dired-preview-start 'function-executed nil))
@@ -746,6 +771,8 @@ the preview with `dired-preview-delay' of idleness."
   "Enable Dired preview."
   (unless (eq major-mode 'dired-mode)
     (user-error "Can only use `dired-preview' in Dired"))
+  (when (>= emacs-major-version 29)
+    (setq-local other-window-scroll-default #'dired-preview-get-first-window))
   (add-hook 'post-command-hook #'dired-preview-trigger nil :local)
   (dired-preview-trigger :no-delay))
 
